/*
 *
 * Miinaturvat - Minesweeper Dismount v0.1 by Heikki Kernen
 *
 */

const int SCRIPT_GAME_VERSION = int(0x000400);

const int MAX_TIME = 60000;
const int LIMB_BONUS = 5000;
int timeLeft = 0;

float SCREEN_CENTER_X;
float SCREEN_CENTER_Y;

uint mTime;
uint gravityTimer = 0;
int screenWidth, screenHeight;

UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

PhysicsObject *col1, *col2;

bool finished = false;
bool aborted = false;

uint test = 0;

bool headUsed = false;
bool rHandUsed = false;
bool lHandUsed = false;
bool lFootUsed = false;
bool rFootUsed = false;

bool mine1 = false;
bool mine2 = false;
bool mine3 = false;
bool mine4 = false;
bool mine5 = false;

bool release1 = false;
bool release2 = false;
bool release3 = false;
bool release4 = false;
bool release5 = false;

bool minesSet = false;

float xGrav = 0;
float yGrav = 0;
float zGrav = -1.5;

uint mine1loc = 0;
uint mine2loc = 0;
uint mine3loc = 0;
uint mine4loc = 0;
uint mine5loc = 0;

// Called once on level initialization.
void onInit()
{
	screenWidth = getScreenWidth();
	screenHeight = getScreenHeight();

	SCREEN_CENTER_X = float(screenWidth / 2);
	SCREEN_CENTER_Y = float(screenHeight / 2);

	medFont = getFont("medium");
	smallFont = getFont("small");
	medFontHeight = medFont->getFontHeight();
	smallFontHeight = smallFont->getFontHeight();

	setGravity(0,0,-1.5);

	head = findObject("Head");
	neck = findObject("Neck");
	trunk = findObject("Trunk");
	rthigh = findObject("RightThigh");
	lthigh = findObject("LeftThigh");
	rarm = findObject("RightArm");
	larm = findObject("LeftArm");
	rfore = findObject("RightForearm");
	lfore = findObject("LeftForearm");
	rcalf = findObject("RightCalf");
	lcalf = findObject("LeftCalf");
	rfoot = findObject("RightFoot");
	lfoot = findObject("LeftFoot");

	for (uint a = 1; a<10; a++) {
		for (uint b = 1; b<4; b++) {
			for (uint c = 0; c<10; c++) {
			test = 9000 + (a * 100) + (b * 10) + c;
			getObject(test)->removeFromPhysicsWorld();
			// had to double the number of particles, quick hack to avoid messing with the IDs
			test += 90000;
			getObject(test)->removeFromPhysicsWorld();
			}
		}
	}

	revertColors();
}

void revertColors() {
	head->setColor(1,0.7,0.5);
	neck->setColor(1,0.7,0.5);
	trunk->setColor(0,0.4,0);
	rthigh->setColor(0,0.4,0);
	lthigh->setColor(0,0.4,0);
	rarm->setColor(0,0.4,0);
	larm->setColor(0,0.4,0);
	rfore->setColor(0,0.4,0);
	lfore->setColor(0,0.4,0);
	rcalf->setColor(0.4,0.1,0.1);
	lcalf->setColor(0.4,0.1,0.1);
	rfoot->setColor(0.4,0.1,0.1);
	lfoot->setColor(0.4,0.1,0.1);
}


void checkTime(uint time) {
	timeLeft = MAX_TIME - int(time);
	if (timeLeft < 0) timeLeft = 0;
	if (timeLeft == 0 && !aborted)
	{
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}
}

void releaseMines (uint id) {
	uint x = uint(id / 10);
	uint y = id % 10;
	for (uint mine = 0; mine < 10; mine++) {
		test = 9000 + (x * 100) + (y * 10) + mine;
		getObject(test)->createToPhysicsWorld();
		// again a quick hack
		test += 90000;
		getObject(test)->createToPhysicsWorld();
	}
	test = 800 + id;

	float x2, y2, z2 = 0;
	getObject(test)->getPosition(x2,y2,z2);
	float mineX = x2;
	float mineY = y2;
	trunk->getPosition(x2,y2,z2);

	zGrav = 3.0;

	if (mineX > x2) xGrav = -1.0;
	else xGrav = 1.0;

	if (mineY > y2) yGrav = -1.0;
	else yGrav = 1.0;

	getObject(test)->removeFromPhysicsWorld();

	gravityTimer = mTime + 500;

}

void drawTime()
{
	const float x = 100, y = 70;
	int fracs = (timeLeft / 10) % 100;
	int seconds = timeLeft / 1000;
	bstr fracsStr = (fracs < 10 ? "0" : "") + bstrFormat(fracs);

	medFont->drawString(x, y, UIFONT_ALIGN_HORIZ_CENTER, ":");
	medFont->drawString(x - 4, y, UIFONT_ALIGN_RIGHT, bstrFormat(seconds));
	medFont->drawString(x + 4, y, UIFONT_ALIGN_DEFAULT, fracsStr);
	smallFont->drawString(x, y + float(medFontHeight), UIFONT_ALIGN_HORIZ_CENTER, "Time Left");
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
	mTime = time;

	float x,y,z = 0;	
	trunk->getPosition(x,y,z);	

	if (z<-25 && !finished) {
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}


	if (!finished && !aborted) checkTime(time);

	if (minesTriggered() == 5) {
		finished = true;
	}

	if (!minesSet) {
		if (x>-25) setMineLocations();
	}

	if (release1) {
	releaseMines(mine1loc);
	release1 = false;
	}
	if (release2) {
	releaseMines(mine2loc);
	release2 = false;
	}
	if (release3) {
	releaseMines(mine3loc);
	release3 = false;
	}
	if (release4) {
	releaseMines(mine4loc);
	release4 = false;
	}
	if (release5) {
	releaseMines(mine5loc);
	release5 = false;
	}

	if (gravityTimer < mTime) {
		zGrav = -1.5;
		xGrav = 0;
		yGrav = 0;
	}

	setGravity(xGrav,yGrav,zGrav);

	revertColors();
}

// Called when the game view is rendered.
// Might happen more or less frequently than update calls.
void onRender()
{
	if (mTime < 5000) {
		float a = 1.0;
		if (mTime > 4000) a = float(5000-mTime) / 1000.0;
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y / 3, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Miinaturvat - Minesweeper Dismount v0.1", 1.0,1.0,1.0,a);
		smallFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y / 3 + 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Clear the minefield before someone gets hurt!", 1.0,1.0,1.0,a);
	}

	drawTime();

	if (aborted) {
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You failed!", 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X+1, SCREEN_CENTER_Y+1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You failed!", 1.0,0.0,0.0,1.0);
	}

	smallFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y / 6, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Mines remaining: " + bstrFormat(5 - minesTriggered()), 1.0,1.0,1.0,1.0);

	if (finished) {
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Clear!", 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Clear!", 0,1.0,0,1.0);

		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Time score: " + bstrFormat(timeLeft), 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 31, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Time score: " + bstrFormat(timeLeft), 1.0,1.0,1.0,1.0);

		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 60, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Limb score: " + bstrFormat(numberOfLimbs() * LIMB_BONUS), 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 61, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Limb score: " + bstrFormat(numberOfLimbs() * LIMB_BONUS), 1.0,1.0,1.0,1.0);

		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 90, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Total score: " + bstrFormat(numberOfLimbs() * LIMB_BONUS + timeLeft), 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 91, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Total score: " + bstrFormat(numberOfLimbs() * LIMB_BONUS + timeLeft), 0.0,1.0,0.0,1.0);
	}

}
// Called when there is a collision between the two given objects.
// Returns true to create contacts or false to ignore collision.
bool onCollision(PhysicsObject *o1, PhysicsObject *o2)
{
	col1 = o1;
	col2 = o2;

	if (!mine1 && playerHit(bstrFormat(mine1loc))) {
		checkLimb();
		release1 = true;
		mine1 = true;
	}

	if (!mine2 && playerHit(bstrFormat(mine2loc))) {
		checkLimb();
		release2 = true;
		mine2 = true;
	}

	if (!mine3 && playerHit(bstrFormat(mine3loc))) {
		checkLimb();
		release3 = true;
		mine3 = true;
	}

	if (!mine4 && playerHit(bstrFormat(mine4loc))) {
		checkLimb();
		release4 = true;
		mine4 = true;
	}

	if (!mine5 && playerHit(bstrFormat(mine5loc))) {
		checkLimb();
		release5 = true;
		mine5 = true;
	}

	return true;
}

void checkLimb() {
	if (col1 == head || col2 == head) headUsed = true;
	if (col1 == lfore || col2 == lfore) lHandUsed = true;
	if (col1 == rfore || col2 == rfore) rHandUsed = true;
	if (col1 == lfoot || col2 == lfoot) lFootUsed = true;
	if (col1 == rfoot || col2 == rfoot) rFootUsed = true;
}

int minesTriggered () {
	int counter = 0;
	if (mine1) counter++;
	if (mine2) counter++;
	if (mine3) counter++;
	if (mine4) counter++;
	if (mine5) counter++;
	return counter;
}

bool playerHit(bstr name) {
	return ((col1->getType() == PHYSOBJ_RAGDOLL_BODY && col2->getName() == name) || (col2->getType() == PHYSOBJ_RAGDOLL_BODY && col1->getName() == name));
}

int numberOfLimbs() {
	int counter = 0;
	if (headUsed) counter++;
	if (rFootUsed) counter++;
	if (lFootUsed) counter++;
	if (lHandUsed) counter++;
	if (rHandUsed) counter++;
	return counter;
}

void setMineLocations() {
	uint ms = mTime % 1000;
	
	mine1loc = 11;
	mine2loc = 32;
	mine3loc = 53;
	mine4loc = 72;
	mine5loc = 91;
	if (ms > 100) {
	mine1loc = 21;
	mine2loc = 33;
	mine3loc = 51;
	mine4loc = 62;
	mine5loc = 83;
	}
	if (ms > 200) {
	mine1loc = 22;
	mine2loc = 41;
	mine3loc = 52;
	mine4loc = 63;
	mine5loc = 93;
	}
	if (ms > 300) {
	mine1loc = 11;
	mine2loc = 13;
	mine3loc = 62;
	mine4loc = 91;
	mine5loc = 93;
	}
	if (ms > 400) {
	mine1loc = 11;
	mine2loc = 43;
	mine3loc = 52;
	mine4loc = 61;
	mine5loc = 93;
	}
	if (ms > 500) {
	mine1loc = 11;
	mine2loc = 33;
	mine3loc = 51;
	mine4loc = 62;
	mine5loc = 81;
	}
	if (ms > 600) {
	mine1loc = 12;
	mine2loc = 33;
	mine3loc = 41;
	mine4loc = 73;
	mine5loc = 81;
	}
	if (ms > 700) {
	mine1loc = 23;
	mine2loc = 33;
	mine3loc = 31;
	mine4loc = 62;
	mine5loc = 91;
	}
	if (ms > 800) {
	mine1loc = 11;
	mine2loc = 21;
	mine3loc = 43;
	mine4loc = 61;
	mine5loc = 82;
	}
	if (ms > 900) {
	mine1loc = 22;
	mine2loc = 41;
	mine3loc = 73;
	mine4loc = 81;
	mine5loc = 92;
	}

	minesSet = true;

}

